package eswtshowcase;

import eswtshowcase.modules.BrowserDemoModule;
import eswtshowcase.modules.ButtonSliderCanvasDemoModule;
import eswtshowcase.modules.ComboMessageBoxDemoModule;
import eswtshowcase.modules.CommandDemoModule;
import eswtshowcase.modules.DemoModule;
import eswtshowcase.modules.DemoModuleGroup;
import eswtshowcase.modules.DialogDemoModule;
import eswtshowcase.modules.InputDemoModule;
import eswtshowcase.modules.ListBoxDemoModule;
import eswtshowcase.modules.ListDemoModule;
import eswtshowcase.modules.SortedListDemoModule;
import eswtshowcase.modules.StaticAndLayoutsDemoModule;
import eswtshowcase.modules.TableDemoModule;
import eswtshowcase.modules.TreeDemoModule;
import javax.microedition.midlet.MIDlet;
import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.ercp.swt.mobile.ListBox;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

/**
 * Main menu screen
 */
public class MainMenuScreen implements Runnable, SelectionListener {
    private MIDlet midlet;
    
    private Display display;
    protected Shell mainShell;
    private Composite mainComposite;
    private ListBox list;
    
    private Command exitCommand;
    
    private DemoModuleListBoxItem[] modules;
    
    private boolean running;
    
    public MainMenuScreen(MIDlet midlet){
        this.midlet = midlet;      
        
        initModules();
    }
    
    public void run(){
        running = true;
        display = new Display();
        
        mainShell = new Shell(display, SWT.NONE);
        mainShell.setLayout(new FillLayout());
        mainShell.setText("eSWT Showcase");
        
        mainShell.open();
        
        mainComposite = new Composite(mainShell, SWT.NONE);
        mainComposite.setLayout(new FillLayout(SWT.VERTICAL));
        mainShell.layout();
        
        exitCommand = new Command(mainShell, Command.EXIT, 0);
        exitCommand.setText("Exit");
        exitCommand.addSelectionListener(this);   

        list = new ListBox(mainComposite, SWT.SINGLE | SWT.V_SCROLL, ListBox.LB_STYLE_2LINE_ITEM);
        list.addSelectionListener(this);
        list.setDataModel(modules);
        
        mainComposite.layout();
        
        //the main loop
        while(running && !mainShell.isDisposed()){
            if(!display.readAndDispatch()){
                display.sleep();
            }
        }
        
        display.dispose();
        midlet.notifyDestroyed();
    }
    
    public void widgetSelected(SelectionEvent e){
        if(e.widget.equals(exitCommand)){
            exit();
        } else if(e.widget.equals(list)){
        }
    }
    
    public void widgetDefaultSelected(SelectionEvent e){
        if(e.widget.equals(list)){
            int focusIndex = list.getFocusIndex();
            if(0 <= focusIndex && focusIndex < modules.length){
                modules[focusIndex].runDemo(mainShell);
            }
        }
        
    }
    
    private void exit(){
        running = false;
        display.wake();
    }
    
    private void initModules(){
        //because the lists are in a group we'll create it here to make
        //the code prettier
        DemoModule[] listDemoModules = new DemoModule[] { 
            new ListDemoModule(), 
            new SortedListDemoModule(),
            new ListBoxDemoModule()};
        DemoModuleGroup listDemos = new DemoModuleGroup("Lists", 
                "List, SortedList, ListBox", listDemoModules);
        
        
        //create the module list
        modules = new DemoModuleListBoxItem[]{ 
            new DemoModuleListBoxItem(listDemos),
            new DemoModuleListBoxItem(new DialogDemoModule()),
            new DemoModuleListBoxItem(new ComboMessageBoxDemoModule()),
            new DemoModuleListBoxItem(new TreeDemoModule()),
            new DemoModuleListBoxItem(new InputDemoModule()),
            new DemoModuleListBoxItem(new TableDemoModule()),
            new DemoModuleListBoxItem(new ButtonSliderCanvasDemoModule()),
            new DemoModuleListBoxItem(new CommandDemoModule()),
            new DemoModuleListBoxItem(new StaticAndLayoutsDemoModule()),
            new DemoModuleListBoxItem(new BrowserDemoModule())
        };
    }
}
