package eswtshowcase.modules;

import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.ercp.swt.mobile.QueryDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

/**
 * A demonstration of the Browser component. Goes to www.nokia.com where from
 * the user can navigate further
 */
public class BrowserDemoModule implements DemoModule, SelectionListener{
    private Shell demoShell;
    private Composite demoComposite;
    private Command closeCommand;
    private Command backCommand;
    private Command forwardCommand;
    private Command goToWebPageCommand;
    
    private Browser browser;
    
    public String getTitle(){
        return "Browser";
    }
    
    public String getDescription(){
        return "Demonstrates the Browser component";
    }
    
    public void runDemo(Shell parent){
        demoShell = new Shell(SWT.MAX);
        demoShell.setText("Browser");
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));
        
        demoComposite = new Composite(demoShell, SWT.NONE);
        demoComposite.setLayout(new FillLayout(SWT.VERTICAL));
        demoShell.layout();
        
        closeCommand = new Command(demoShell, Command.EXIT, 0);
        closeCommand.setText("Close");
        closeCommand.addSelectionListener(this);
        
        backCommand = new Command(demoShell, Command.BACK, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);
        
        forwardCommand = new Command(demoShell, Command.OK, 0);
        forwardCommand.setText("Forward");
        forwardCommand.addSelectionListener(this);
        
        goToWebPageCommand = new Command(demoShell, Command.OK, 0);
        goToWebPageCommand.setText("Go To Web Page");
        goToWebPageCommand.addSelectionListener(this);
        
        //create the browser
        browser = new Browser(demoComposite, SWT.NONE);
        
        demoComposite.layout();
        demoShell.open();
        demoShell.setActive();
        
        //browse to nokia.com
        browser.setUrl("http://forumnokia.mobi");
    }
    
    public void widgetSelected(SelectionEvent e){
        if(e.widget.equals(closeCommand)){
            close();
        } else if(e.widget.equals(backCommand)){
            if(browser.isBackEnabled()){
                browser.back();
            }
        } else if(e.widget.equals(forwardCommand)){
            if(browser.isForwardEnabled()){
                browser.forward();
            }
        } else if(e.widget.equals(goToWebPageCommand)){
            String url = getUrl();
            if(url != null){
                browser.setUrl(url);
            }
        }
    }
    
    public void widgetDefaultSelected(SelectionEvent e){
        
    }
    
    private void close(){
        //stop loading the current page
        browser.stop();
        
        demoShell.dispose();
        demoShell = null;
    }
    
    private String getUrl(){
        QueryDialog dlg = new QueryDialog(demoShell);
        dlg.setPromptText("Enter URL:", browser.getUrl());
        String url = dlg.open();
        
        if(url != null){
            //make the url properly formatted
            if(!url.startsWith("http://")){
                url = "http://" + url;
            }
        }
        
        return url;
    }
}
