package eswtshowcase.modules;

import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.ercp.swt.mobile.QueryDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
//import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.FontDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

/**
 * A demonstration of the different ready-made Dialogs.
 */
public class DialogDemoModule implements DemoModule, SelectionListener {
    private Shell demoShell;
    private Composite demoComposite;
    
    private Command backCommand;
    
    private Label colorLabel;
    private Color color;
    private Button colorButton;
    
    private Label directoryText;
    private Button directoryButton;
    
    private Label fileText;
    private Button fileButton;
    
    private Label fontText;
    private Button fontButton;
    
    private Label queryText;
    private Button queryButton;
    
    public void runDemo(Shell parent){
        demoShell = new Shell(SWT.MAX);
        demoShell.setText("Dialog Demo");
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));
        
        demoComposite = new Composite(demoShell, SWT.NONE);
        demoComposite.setLayout(new FormLayout());
        demoShell.layout();
        
        backCommand = new Command(demoShell, Command.EXIT, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);
        
        colorLabel = new Label(demoComposite, SWT.SHADOW_NONE|SWT.CENTER);
        colorLabel.setText("Not yet chosen");
        colorLabel.setLayoutData(createLayoutDataForText(0));
        
        colorButton = new Button(demoComposite, SWT.PUSH);
        colorButton.setText("Color");
        colorButton.addSelectionListener(this);
        colorButton.setLayoutData(createLayoutDataForButton(0));
        
        directoryText = new Label(demoComposite, SWT.SHADOW_NONE|SWT.CENTER);
        directoryText.setText("Not yet chosen");
        directoryText.setLayoutData(createLayoutDataForText(20));
        
        directoryButton = new Button(demoComposite, SWT.PUSH);
        directoryButton.setText("Dir");
        directoryButton.addSelectionListener(this);
        directoryButton.setLayoutData(createLayoutDataForButton(20));
        
        fileText = new Label(demoComposite, SWT.SHADOW_NONE|SWT.CENTER);
        fileText.setText("Not yet chosen");
        fileText.setLayoutData(createLayoutDataForText(40));
        
        fileButton = new Button(demoComposite, SWT.PUSH|SWT.FLAT);
        fileButton.setText("File");
        fileButton.addSelectionListener(this);
        fileButton.setLayoutData(createLayoutDataForButton(40));
        
        fontText = new Label(demoComposite, SWT.SHADOW_NONE|SWT.CENTER);
        fontText.setText("Not yet chosen");
        fontText.setLayoutData(createLayoutDataForText(60));
        
        fontButton = new Button(demoComposite, SWT.PUSH);
        fontButton.setText("Font");
        fontButton.addSelectionListener(this);
        fontButton.setLayoutData(createLayoutDataForButton(60));
        
        queryText = new Label(demoComposite, SWT.SHADOW_NONE|SWT.CENTER);
        queryText.setText("Not yet chosen");
        queryText.setLayoutData(createLayoutDataForText(80));
        
        queryButton = new Button(demoComposite, SWT.PUSH);
        queryButton.setText("Query");
        queryButton.addSelectionListener(this);
        queryButton.setLayoutData(createLayoutDataForButton(80));
        
        demoComposite.layout();
        demoShell.open();
        demoShell.setActive();
    }
    
    public String getTitle(){
        return "Dialogs";
    }
    
    public String getDescription(){
        return "Demonstrates all the Dialogs";
    }
    
    public void widgetSelected(SelectionEvent e){
        if(e.widget.equals(backCommand)){
            back();
        } else if(e.widget.equals(colorButton)){
            chooseColor();
        } else if(e.widget.equals(directoryButton)){
            chooseDirectory();
        } else if(e.widget.equals(fileButton)){
            chooseFile();
        } else if(e.widget.equals(fontButton)){
            chooseFont();
        } else if(e.widget.equals(queryButton)){
            chooseText();
        }
    }
    
    public void widgetDefaultSelected(SelectionEvent e){
        
    }
    
    private void back(){
        if(color != null){
            color.dispose();
            color = null;
        }
        
        demoShell.dispose();
        demoShell = null;
    }
    
    private void chooseColor(){
        //create a ColorDialog and open it
        ColorDialog dialog = new ColorDialog(demoShell);
        RGB result = dialog.open();
        //if the user canceled the result is null
        if(result == null){
            return;
        } else {
            //display the chosen color in the color label
            colorLabel.setText("");
            
            if(color != null){
                color.dispose();
                color = null;
            }            
            color = new Color(demoShell.getDisplay(), result);
            colorLabel.setBackground(color);
        }
    }
    
    private void chooseDirectory(){
        //create a DirectoryDialog and open it
        DirectoryDialog dialog = new DirectoryDialog(demoShell);
        String result = dialog.open();
        //if the user canceled the result is null
        if(result == null){
            return;
        } else {
            //display the choice
            directoryText.setText(result);
        }
    }
    
    private void chooseFile(){
        //create a FileDialog and open it
        FileDialog dialog = new FileDialog(demoShell);
        String result = dialog.open();
        //check if the user canceled
        if(result == null){
            return;
        } else {
            //display the result in the correct label
            fileText.setText(result);
        }
    }
    
    private void chooseFont(){
        //construct a FontDialog and open it
        FontDialog dialog = new FontDialog(demoShell);
        FontData result = dialog.open();
        //the result is null if the user canceled
        if(result == null){
            return;
        } else {
            //FontDialog returns FontData so we'll have to take the name for display
            fontText.setText(result.getName());
        }
    }
    
    private void chooseText(){
        //create a QueryDialog and set the prompt texts
        QueryDialog dialog = new QueryDialog(demoShell);
        dialog.setPromptText("Enter Text:", "Text");
        String result = dialog.open();
        //the user canceled if the result is null
        if(result == null){
            return;
        } else {
            queryText.setText(result);
        }
    }
    
    private FormData createLayoutDataForText(int y){
        FormData formData = new FormData();
        formData.top = new FormAttachment(y);
        formData.left = new FormAttachment(0);
        formData.right = new FormAttachment(70);
        formData.bottom = new FormAttachment(y+20);
        return formData;
    }
    
    private FormData createLayoutDataForButton(int y){
        FormData formData = new FormData();
        formData.top = new FormAttachment(y, 3);
        formData.left = new FormAttachment(70, 3);
        formData.right = new FormAttachment(100, -3);
        formData.bottom = new FormAttachment(y+20, -3);
        return formData;
    }
}
