/*******************************************************************************
 * Copyright  2010 Nokia Corporation. All rights reserved.
 * Nokia and Nokia Connecting People are registered trademarks of Nokia Corporation.
 * Java and all Java-based marks are trademarks or registered trademarks of Sun Microsystems, Inc.
 * Other product and company names mentioned herein may be trademarks or trade names of their respective owners.
 *
 * Subject to the conditions below, you may, without charge:
 *
 *   Use, copy, modify and/or merge copies of this software and associated documentation files (the Software)
 *
 *   Publish, distribute, sub-license and/or sell new software derived from or incorporating the Software.
 *
 *
 *
 * This license, unmodified, shall be included with all copies or
 * substantial portions of the Software that are distributed in source code form.
 *
 * The Software cannot constitute the primary value of any new software derived from or incorporating the Software.
 *
 * Any person dealing with the Software shall not misrepresent the source of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************/
package eswtshowcase.modules;

import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.ercp.swt.mobile.QueryDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

/**
 * A demonstration of the Browser component. Goes to www.nokia.com where from
 * the user can navigate further
 */
public class BrowserDemoModule implements DemoModule, SelectionListener
{
    private Shell demoShell;
    private Composite demoComposite;
    private Command closeCommand;
    private Command backCommand;
    private Command forwardCommand;
    private Command goToWebPageCommand;

    private Browser browser;

    public String getTitle()
    {
        return "Browser";
    }

    public String getDescription()
    {
        return "Demonstrates the Browser component";
    }

    public void runDemo(Shell parent)
    {
        demoShell = new Shell(SWT.MAX);
        demoShell.setText("Browser");
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));

        demoComposite = new Composite(demoShell, SWT.NONE);
        demoComposite.setLayout(new FillLayout(SWT.VERTICAL));
        demoShell.layout();

        closeCommand = new Command(demoShell, Command.EXIT, 0);
        closeCommand.setText("Close");
        closeCommand.addSelectionListener(this);

        backCommand = new Command(demoShell, Command.BACK, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);

        forwardCommand = new Command(demoShell, Command.OK, 0);
        forwardCommand.setText("Forward");
        forwardCommand.addSelectionListener(this);

        goToWebPageCommand = new Command(demoShell, Command.OK, 0);
        goToWebPageCommand.setText("Go To Web Page");
        goToWebPageCommand.addSelectionListener(this);

        //create the browser
        browser = new Browser(demoComposite, SWT.NONE);

        demoComposite.layout();
        demoShell.open();
        demoShell.setActive();

        //browse to nokia.com
        browser.setUrl("http://forumnokia.mobi");
    }

    public void widgetSelected(SelectionEvent e)
    {
        if(e.widget.equals(closeCommand))
        {
            close();
        }
        else if(e.widget.equals(backCommand))
        {
            if(browser.isBackEnabled())
            {
                browser.back();
            }
        }
        else if(e.widget.equals(forwardCommand))
        {
            if(browser.isForwardEnabled())
            {
                browser.forward();
            }
        }
        else if(e.widget.equals(goToWebPageCommand))
        {
            String url = getUrl();
            if(url != null)
            {
                browser.setUrl(url);
            }
        }
    }

    public void widgetDefaultSelected(SelectionEvent e)
    {

    }

    private void close()
    {
        //stop loading the current page
        browser.stop();

        demoShell.dispose();
        demoShell = null;
    }

    private String getUrl()
    {
        QueryDialog dlg = new QueryDialog(demoShell);
        dlg.setPromptText("Enter URL:", browser.getUrl());
        String url = dlg.open();

        if(url != null)
        {
            //make the url properly formatted
            if(!url.startsWith("http://"))
            {
                url = "http://" + url;
            }
        }

        return url;
    }
}
