/*******************************************************************************
 * Copyright  2010 Nokia Corporation. All rights reserved.
 * Nokia and Nokia Connecting People are registered trademarks of Nokia Corporation.
 * Java and all Java-based marks are trademarks or registered trademarks of Sun Microsystems, Inc.
 * Other product and company names mentioned herein may be trademarks or trade names of their respective owners.
 *
 * Subject to the conditions below, you may, without charge:
 *
 *   Use, copy, modify and/or merge copies of this software and associated documentation files (the Software)
 *
 *   Publish, distribute, sub-license and/or sell new software derived from or incorporating the Software.
 *
 *
 *
 * This license, unmodified, shall be included with all copies or
 * substantial portions of the Software that are distributed in source code form.
 *
 * The Software cannot constitute the primary value of any new software derived from or incorporating the Software.
 *
 * Any person dealing with the Software shall not misrepresent the source of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************/
package eswtshowcase.modules;

import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
//import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

/**
 *
 */
public class CommandDemoModule implements DemoModule, SelectionListener
{
    private Shell demoShell;
    private Composite demoComposite;

    private Command backCommand;

    private Command firstCommand;
    private Command secondCommand;
    private Command commandGroup;
    private Command firstGroupCommand;
    private Command secondGroupCommand;
    private Command thirdGroupCommand;
    private Command thirdCommand;

    public String getTitle()
    {
        return "Commands";
    }

    public String getDescription()
    {
        return "Demonstrates Commands";
    }

    public void runDemo(Shell parent)
    {
        demoShell = new Shell(SWT.MAX);
        demoShell.setText("Commands");
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));

        demoComposite = new Composite(demoShell, SWT.NONE);
        demoComposite.setLayout(new FillLayout(SWT.VERTICAL));
        demoShell.layout();

        //Create a Back command and add this as a selection listener
        //to get notified when the user uses the back button
        backCommand = new Command(demoShell, Command.EXIT, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);

        firstCommand = new Command(demoShell, Command.SELECT, 1);
        firstCommand.setText("First");
        firstCommand.addSelectionListener(this);

        secondCommand = new Command(demoShell, Command.SELECT, 1);
        secondCommand.setText("Second");
        secondCommand.addSelectionListener(this);

        //Command groups are created by specifying the command type parameter
        //Command.COMMANDGROUP
        commandGroup = new Command(demoShell, Command.COMMANDGROUP, 1);
        commandGroup.setText("Group");
        commandGroup.addSelectionListener(this);

        //Adding commands to the command group works by setting the parent
        //of the command as the command group command. This works only if
        //the parent's type is Command.COMMANDGROUP
        firstGroupCommand = new Command(commandGroup, Command.SELECT, 2);
        firstGroupCommand.setText("First");
        firstGroupCommand.addSelectionListener(this);

        secondGroupCommand = new Command(commandGroup, Command.SELECT, 2);
        secondGroupCommand.setText("Second");
        secondGroupCommand.addSelectionListener(this);

        thirdGroupCommand = new Command(commandGroup, Command.SELECT, 2);
        thirdGroupCommand.setText("Third");
        thirdGroupCommand.addSelectionListener(this);

        thirdCommand = new Command(demoShell, Command.SELECT, 1);
        thirdCommand.setText("Third");
        thirdCommand.addSelectionListener(this);

        demoComposite.layout();
        demoShell.open();
        demoShell.setActive();
    }

    public void widgetSelected(SelectionEvent e)
    {
        //here we can test whether it's command x that's selected
        if(e.widget.equals(backCommand))
        {
            back();
        }
        else if(e.widget.equals(firstCommand))
        {
            //add command handling code here
        }
        else if(e.widget.equals(secondCommand))
        {

        }
        else if(e.widget.equals(firstGroupCommand))
        {

        }
        else if(e.widget.equals(secondGroupCommand))
        {

        }
        else if(e.widget.equals(thirdGroupCommand))
        {

        }
        else if(e.widget.equals(thirdCommand))
        {

        }
    }

    public void widgetDefaultSelected(SelectionEvent e)
    {
        //this doesn't get called for commands
    }

    private void back()
    {
        demoShell.dispose();
        demoShell = null;
    }
}
