/*******************************************************************************
 * Copyright  2010 Nokia Corporation. All rights reserved.
 * Nokia and Nokia Connecting People are registered trademarks of Nokia Corporation.
 * Java and all Java-based marks are trademarks or registered trademarks of Sun Microsystems, Inc.
 * Other product and company names mentioned herein may be trademarks or trade names of their respective owners.
 *
 * Subject to the conditions below, you may, without charge:
 *
 *   Use, copy, modify and/or merge copies of this software and associated documentation files (the Software)
 *
 *   Publish, distribute, sub-license and/or sell new software derived from or incorporating the Software.
 *
 *
 *
 * This license, unmodified, shall be included with all copies or
 * substantial portions of the Software that are distributed in source code form.
 *
 * The Software cannot constitute the primary value of any new software derived from or incorporating the Software.
 *
 * Any person dealing with the Software shall not misrepresent the source of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************/
package eswtshowcase.modules;

import eswtshowcase.DemoModuleListBoxItem;
import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.ercp.swt.mobile.ListBox;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;


/**
 * A class to simplify having many DemoModules grouped together
 */
public class DemoModuleGroup implements DemoModule, SelectionListener, ControlListener
{
    protected Shell demoShell;

    private DemoModule[] modules;
    private ListBox moduleListBox;

    private String title;
    private String description;

    private Shell parentShell;

    public DemoModuleGroup(String title, String description, DemoModule[] modules)
    {
        this.title = title;
        this.description = description;

        this.modules = modules;
    }

    public void runDemo(Shell parent)
    {
        parentShell = parent;
        parentShell.addControlListener(this);
        demoShell = new Shell(parent);
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));
        demoShell.setText(title);

        Composite demoComposite = new Composite(demoShell, SWT.NONE);
        demoComposite.setLayout(new FillLayout(SWT.VERTICAL));


        Command backCommand = new Command(demoShell, Command.BACK, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);

        //create a list based on the data in modules
        DemoModuleListBoxItem[] moduleListBoxItems = new DemoModuleListBoxItem[modules.length];
        for(int i=0; i<moduleListBoxItems.length; i++)
        {
            moduleListBoxItems[i] = new DemoModuleListBoxItem(modules[i]);
        }

        moduleListBox = new ListBox(demoComposite, SWT.SINGLE, ListBox.LB_STYLE_2LINE_ITEM);
        moduleListBox.addSelectionListener(this);
        moduleListBox.setDataModel(moduleListBoxItems);

        demoShell.layout();
        demoShell.open();
    }

    public void widgetSelected(SelectionEvent e)
    {
        if(e.widget instanceof Command)
        {
            back();
        }
    }

    public void widgetDefaultSelected(SelectionEvent e)
    {
        if(e.widget.equals(moduleListBox))
        {
            int selection = moduleListBox.getFocusIndex();
            modules[selection].runDemo(demoShell);
        }
    }
    public void controlMoved(ControlEvent e)
    {

    }
    public void controlResized(ControlEvent e)
    {
        if(demoShell != null)
        {
            demoShell.setLocation(
                demoShell.getParent().getClientArea().width/2 - demoShell.getBounds().width/2,
                demoShell.getParent().toDisplay(0,0).y + demoShell.getParent().getClientArea().height - demoShell.getBounds().height);
        }
    }

    private void back()
    {
        demoShell.dispose();
        demoShell = null;
    }

    public String getTitle()
    {
        return title;
    }

    public String getDescription()
    {
        return description;
    }
}