/*******************************************************************************
 * Copyright  2010 Nokia Corporation. All rights reserved.
 * Nokia and Nokia Connecting People are registered trademarks of Nokia Corporation.
 * Java and all Java-based marks are trademarks or registered trademarks of Sun Microsystems, Inc.
 * Other product and company names mentioned herein may be trademarks or trade names of their respective owners.
 *
 * Subject to the conditions below, you may, without charge:
 *
 *   Use, copy, modify and/or merge copies of this software and associated documentation files (the Software)
 *
 *   Publish, distribute, sub-license and/or sell new software derived from or incorporating the Software.
 *
 *
 *
 * This license, unmodified, shall be included with all copies or
 * substantial portions of the Software that are distributed in source code form.
 *
 * The Software cannot constitute the primary value of any new software derived from or incorporating the Software.
 *
 * Any person dealing with the Software shall not misrepresent the source of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************/
package eswtshowcase.modules;

import org.eclipse.ercp.swt.mobile.CaptionedControl;
import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.ercp.swt.mobile.ConstrainedText;
import org.eclipse.ercp.swt.mobile.ListBox;
import org.eclipse.ercp.swt.mobile.ListBoxItem;
import org.eclipse.ercp.swt.mobile.TimedMessageBox;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ControlListener;

/**
 * Demonstrates the ListBox component using a custom-made ListBoxItem called
 * Contact that holds some contact info. Also includes a demonstration of a
 * custom-made dialog.
 */
public class ListBoxDemoModule implements DemoModule, SelectionListener
{
    private Shell demoShell;
    private Shell parent;
    private Composite demoComposite;
    private ListBox demoListBox;

    private Command backCommand;
    private Command addContactCommand;
    private Command removeContactsCommand;
    private Command sendSmsCommand;
    private Command changeDetailCommandGroup;
    private Command useTitleCommand;
    private Command usePhoneNumberCommand;
    private Command useEmailAddressCommand;

    //notice that ListBoxes use ListBoxItems instead of Strings
    private Contact[] contacts;

    public void runDemo(Shell parent)
    {
        this.parent = parent;
        contacts = new Contact[]
        {
            new Contact("John Doe", "Executive Sales Manager", "+358401234567", "john.doe@johndoe.net"),
            new Contact("Jane Smith", "CEO", "+1507654321", "jane@thefirm.com")
        };

        demoShell = new Shell(SWT.MAX);
        demoShell.setText("Contact List");
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));

        demoComposite = new Composite(demoShell, SWT.NONE);
        demoComposite.setLayout(new FillLayout(SWT.VERTICAL));
        demoShell.layout();

        backCommand = new Command(demoShell, Command.EXIT, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);

        addContactCommand = new Command(demoShell, Command.SELECT, 1);
        addContactCommand.setText("Add Contact");
        addContactCommand.addSelectionListener(this);

        removeContactsCommand = new Command(demoShell, Command.SELECT, 1);
        removeContactsCommand.setText("Remove Contact(s)");
        removeContactsCommand.addSelectionListener(this);

        sendSmsCommand = new Command(demoShell, Command.SELECT, 1);
        sendSmsCommand.setText("Send SMS");
        sendSmsCommand.addSelectionListener(this);

        changeDetailCommandGroup = new Command(demoShell, Command.COMMANDGROUP, 1);
        changeDetailCommandGroup.setText("Change Detail Text");
        changeDetailCommandGroup.addSelectionListener(this);

        useTitleCommand = new Command(changeDetailCommandGroup, Command.SELECT, 2);
        useTitleCommand.setText("Use Title");
        useTitleCommand.addSelectionListener(this);
        useTitleCommand.setEnabled(false);

        usePhoneNumberCommand = new Command(changeDetailCommandGroup, Command.SELECT, 2);
        usePhoneNumberCommand.setText("Use Phone Number");
        usePhoneNumberCommand.addSelectionListener(this);

        useEmailAddressCommand = new Command(changeDetailCommandGroup, Command.SELECT, 2);
        useEmailAddressCommand.setText("Use E-mail Address");
        useEmailAddressCommand.addSelectionListener(this);

        //initialize a multiple selection ListBox with the
        //2-line style (heading and description text), also set the original content
        demoListBox = new ListBox(demoComposite, SWT.MULTI, ListBox.LB_STYLE_2LINE_ITEM);
        demoListBox.setDataModel(contacts);
        demoListBox.addSelectionListener(this);
        demoComposite.layout();
        demoShell.open();
        demoShell.setActive();
    }

    public String getTitle()
    {
        return "Contact List";
    }

    public String getDescription()
    {
        return "Demonstrates the ListBox component";
    }

    public void widgetSelected(SelectionEvent e)
    {
        if(e.widget.equals(backCommand))
        {
            back();
        }
        else if(e.widget.equals(useTitleCommand))
        {
            changeDetailText(Contact.TITLE);
        }
        else if(e.widget.equals(usePhoneNumberCommand))
        {
            changeDetailText(Contact.PHONE);
        }
        else if(e.widget.equals(useEmailAddressCommand))
        {
            changeDetailText(Contact.EMAIL);
        }
        else if(e.widget.equals(addContactCommand))
        {
            addContact();
        }
        else if(e.widget.equals(removeContactsCommand))
        {
            removeContacts();
        }
        else if(e.widget.equals(sendSmsCommand))
        {
            sendSms();
        }
    }

    public void widgetDefaultSelected(SelectionEvent e)
    {

    }

    private void back()
    {
        demoShell.dispose();
        demoShell = null;
        parent.getShell().setActive();
    }

    private void addContact()
    {
        //display our custom dialog that shows a contact entry form
        ContactEntryDialog dialog = new ContactEntryDialog(demoShell);
        Contact newContact = dialog.open();

        //if the user cancelled
        if(newContact == null)
        {
            return;
        }

        //check if the contact is already in the list
        if(alreadyInList(newContact))
        {
            TimedMessageBox error = new TimedMessageBox(demoShell, SWT.ICON_ERROR);
            error.setMessage("Cannot add a duplicate contact.");
            error.open();
        }
        else
        {
            //add it to the data model
            addToList(newContact);
            //update the data model the list uses
            demoListBox.setDataModel(contacts);
            demoListBox.refreshList();
        }
    }

    private void removeContacts()
    {
        int[] selectionIndices = demoListBox.getSelectionIndices();
        //if no items are selected
        if(selectionIndices.length == 0)
        {
            return;
        }

        //remove the items from the data model
        removeFromList(selectionIndices);
        //update the list to use the new data model
        demoListBox.setDataModel(contacts);
        demoListBox.refreshList();
    }

    private void changeDetailText(int newDetail)
    {
        //depending on which detail was selected, enable/disable the right commands
        switch(newDetail)
        {
        case Contact.TITLE:
            useTitleCommand.setEnabled(false);
            usePhoneNumberCommand.setEnabled(true);
            useEmailAddressCommand.setEnabled(true);
            break;
        case Contact.PHONE:
            useTitleCommand.setEnabled(true);
            usePhoneNumberCommand.setEnabled(false);
            useEmailAddressCommand.setEnabled(true);
            break;
        case Contact.EMAIL:
            useTitleCommand.setEnabled(true);
            usePhoneNumberCommand.setEnabled(true);
            useEmailAddressCommand.setEnabled(false);
            break;
        default:
            break;
        }

        //switch all the detail properties to the new detail
        for(int i = 0; i < contacts.length; i++)
        {
            contacts[i].switchDetailProperty(newDetail);
        }

        //since all items have been changed, refresh the whole list
        demoListBox.refreshList();
    }

    private void sendSms()
    {
        //display a simple TimedMessageBox that shows the selection count
        TimedMessageBox box = new TimedMessageBox(demoShell, SWT.ICON_WORKING);
        int selectionCount = demoListBox.getSelectionCount();
        box.setMessage("Sending a message to " + selectionCount +
                       ((selectionCount == 1) ? " contact." : " contacts."));
        box.open();
    }

    //helper method to check whether a Contact is already in the list
    private boolean alreadyInList(Contact contact)
    {
        for(int i = 0; i < contacts.length; i++)
        {
            if(contacts[i].equals(contact))
            {
                return true;
            }
        }

        return false;
    }

    //helper method to add a Contact to the contacts array
    private void addToList(Contact contactToAdd)
    {
        Contact[] newContacts = new Contact[contacts.length+1];
        System.arraycopy(contacts, 0, newContacts, 0, contacts.length);
        newContacts[newContacts.length-1] = contactToAdd;
        contacts = newContacts;
    }

    //helper method to remove multiple Contacts from the contacts array
    private void removeFromList(int[] contactsToRemove)
    {
        Contact[] newContacts = new Contact[contacts.length-contactsToRemove.length];

        int addIndex = 0;
        for(int i=0; i < contacts.length; i++)
        {
            boolean add = true;
            for(int j = 0; j < contactsToRemove.length; j++)
            {
                if(contactsToRemove[j] == i)
                {
                    add = false;
                    break;
                }
            }

            if(add)
            {
                newContacts[addIndex] = contacts[i];
                addIndex++;
            }
        }

        contacts = newContacts;
    }

    /*
     * This is our custom Contact class that extends ListBoxItem so we can use it
     * directly with our ListBox. The main reason this class exists is to
     * simplify switching between detail texts.
     */
    private static class Contact extends ListBoxItem
    {
        String name;
        String title;
        String phoneNumber;
        String emailAddress;

        public static final int TITLE = 0;
        public static final int PHONE = 1;
        public static final int EMAIL = 2;

        public Contact(String name, String title, String phoneNumber, String emailAddress)
        {
            super(title, null, name, null);

            this.name = name;
            this.title = title;
            this.phoneNumber = phoneNumber;
            this.emailAddress = emailAddress;
        }

        public void switchDetailProperty(int property)
        {
            switch(property)
            {
            case Contact.TITLE:
                this.setDetailText(title);
                break;
            case Contact.PHONE:
                this.setDetailText(phoneNumber);
                break;
            case Contact.EMAIL:
                this.setDetailText(emailAddress);
                break;
            default:
                break;
            }
        }

        public boolean equals(Contact contact)
        {
            return this.name.equals(contact.name) &&
                   this.title.equals(contact.title) &&
                   this.phoneNumber.equals(contact.phoneNumber) &&
                   this.emailAddress.equals(contact.emailAddress);
        }
    }

    /*
     * This is our custom dialog form for entering Contact info.
     *
     */
    private static class ContactEntryDialog extends Dialog implements SelectionListener, ControlListener
    {
        Contact result = null;

        Shell dialogShell;
        ScrolledComposite scrolledComposite;
        final Composite c1;
        Command addCommand;
        Command cancelCommand;

        private Shell parentShell;

        Text name;
        Text title;
        ConstrainedText phoneNumber;
        Text emailAddress;

        public ContactEntryDialog(Shell parent)
        {
            super(parent, SWT.PRIMARY_MODAL);
            parentShell = parent;
            parentShell.addControlListener(this);
            setText("Contact Data");

            //create the shell with the style parameters for a dialog
            dialogShell = new Shell(parent, SWT.PRIMARY_MODAL|SWT.DIALOG_TRIM);
            dialogShell.setText("Contact Data");
            dialogShell.setBounds(parent.getClientArea());
            dialogShell.setLocation(parent.toDisplay(0, 0));
            dialogShell.setLayout(new FillLayout(SWT.VERTICAL));

            scrolledComposite = new ScrolledComposite(dialogShell,SWT.V_SCROLL);
            c1 = new Composite(scrolledComposite, SWT.NONE);
            c1.setLayout(new GridLayout(1, false));

            addCommand = new Command(dialogShell, Command.OK, 0);
            addCommand.setText("Add");
            addCommand.addSelectionListener(this);

            cancelCommand = new Command(dialogShell, Command.CANCEL, 0);
            cancelCommand.setText("Cancel");
            cancelCommand.addSelectionListener(this);

            CaptionedControl textControl = new CaptionedControl(c1, SWT.NONE);
            name = new Text(textControl, SWT.SINGLE);
            textControl.setText("Name");
            textControl.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));

            CaptionedControl titleControl = new CaptionedControl(c1, SWT.NONE);
            title = new Text(titleControl, SWT.SINGLE);
            titleControl.setText("Title");
            titleControl.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));

            CaptionedControl phoneNumberControl = new CaptionedControl(c1, SWT.NONE);
            phoneNumber = new ConstrainedText(phoneNumberControl, SWT.NONE, ConstrainedText.PHONENUMBER);
            phoneNumberControl.setText("Phone Number");
            phoneNumberControl.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));

            CaptionedControl emailAddressControl = new CaptionedControl(c1, SWT.NONE);
            emailAddress = new Text(emailAddressControl, SWT.SINGLE);
            emailAddressControl.setText("E-mail Address");
            emailAddressControl.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));

            dialogShell.layout();
            c1.setSize(c1.computeSize(scrolledComposite.getClientArea().width-scrolledComposite.getVerticalBar().getSize().x, -1));
            scrolledComposite.setContent(c1);
        }

        public Contact open()
        {
            dialogShell.open();

            //our own event loop to wait for the user to exit the dialog,
            //otherwise we wouldn't be able to get the result this way
            Display display = getParent().getDisplay();
            while(!dialogShell.isDisposed())
            {
                if(!display.readAndDispatch())
                {
                    display.sleep();
                }
            }

            return result;
        }

        public void widgetSelected(SelectionEvent e)
        {
            if(e.widget.equals(addCommand))
            {
                //construct the result with the data in the form
                result = new Contact(name.getText(), title.getText(),
                                     phoneNumber.getText(), emailAddress.getText());
                //dispose of the shell to allow open() to return
                dialogShell.dispose();
            }
            else if(e.widget.equals(cancelCommand))
            {
                result = null;
                //dispose of the shell to allow open() to return
                dialogShell.dispose();
            }
        }

        public void widgetDefaultSelected(SelectionEvent e)
        {

        }
        public void controlMoved(ControlEvent e)
        {

        }
        public void controlResized(ControlEvent e)
        {
            if(!dialogShell.isDisposed())
            {

                dialogShell.setRedraw(false);
                dialogShell.setBounds(dialogShell.getParent().getClientArea());
                dialogShell.setLocation(dialogShell.getParent().toDisplay(0, 0));
                c1.setSize(c1.computeSize(scrolledComposite.getClientArea().width-scrolledComposite.getVerticalBar().getSize().x, -1));
                c1.layout();
                dialogShell.layout();
                dialogShell.setRedraw(true);
            }
        }

    }
}
