/*******************************************************************************
 * Copyright  2010 Nokia Corporation. All rights reserved.
 * Nokia and Nokia Connecting People are registered trademarks of Nokia Corporation.
 * Java and all Java-based marks are trademarks or registered trademarks of Sun Microsystems, Inc.
 * Other product and company names mentioned herein may be trademarks or trade names of their respective owners.
 *
 * Subject to the conditions below, you may, without charge:
 *
 *   Use, copy, modify and/or merge copies of this software and associated documentation files (the Software)
 *
 *   Publish, distribute, sub-license and/or sell new software derived from or incorporating the Software.
 *
 *
 *
 * This license, unmodified, shall be included with all copies or
 * substantial portions of the Software that are distributed in source code form.
 *
 * The Software cannot constitute the primary value of any new software derived from or incorporating the Software.
 *
 * Any person dealing with the Software shall not misrepresent the source of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************/
package eswtshowcase.modules;

import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

/**
 * Demonstrates the Table component by showing some personal info about people.
 *
 */
public class TableDemoModule implements DemoModule, SelectionListener
{

    private Shell demoShell;
    private Composite demoComposite;
    private Table demoTable;

    private Color white;
    private Color blue;
    private Color red;
    private Color pink;
    private Color green;

    private Command backCommand;
    private Command toggleLinesVisibleCommand;
    private Command toggleHeaderVisibleCommand;

    private boolean linesVisible = true;
    private boolean headerVisible = true;

    public String getTitle()
    {
        return "Table";
    }

    public String getDescription()
    {
        return "Demonstrates the Table component";
    }

    public void runDemo(Shell parent)
    {
        demoShell = new Shell(SWT.MAX);
        demoShell.setText("Table Demo");
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));

        demoComposite = new Composite(demoShell, SWT.NONE);
        demoComposite.setLayout(new FillLayout(SWT.VERTICAL));
        demoShell.layout();

        backCommand = new Command(demoShell, Command.EXIT, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);

        toggleLinesVisibleCommand = new Command(demoShell, Command.SELECT, 1);
        toggleLinesVisibleCommand.setText("Toggle Lines Visible");
        toggleLinesVisibleCommand.addSelectionListener(this);

        toggleHeaderVisibleCommand = new Command(demoShell, Command.SELECT, 1);
        toggleHeaderVisibleCommand.setText("Toggle Header Visible");
        toggleHeaderVisibleCommand.addSelectionListener(this);

        //Create the colors we use, remember to dispose them afterwards!
        white = new Color(demoShell.getDisplay(), 255, 255, 255);
        blue = new Color(demoShell.getDisplay(), 0, 0, 255);
        red = new Color(demoShell.getDisplay(), 255, 0, 0);
        pink = new Color(demoShell.getDisplay(), 255, 150, 150);
        green = new Color(demoShell.getDisplay(), 0, 180, 0);

        //create a multiple selection table with check boxes, also set
        //the lines and header to be visible
        demoTable = new Table(demoComposite, SWT.MULTI|SWT.CHECK);
        demoTable.setLinesVisible(linesVisible);
        demoTable.setHeaderVisible(headerVisible);

        //the columns we want to have
        String[] titles = new String[] {"Name", "Age", "Color", "Food"};
        for(int i = 0; i < titles.length; i++)
        {
            TableColumn column = new TableColumn(demoTable, SWT.LEFT);
            column.setText(titles[i]);
        }

        //create the item
        TableItem john = new TableItem(demoTable, SWT.NONE);
        //set the text with a string array, the data goes into the columns
        //in the order they're entered
        String[] johnInfo = {"John", "32", "Blue", "Pork Chops"};
        john.setText(johnInfo);
        //set the colors to look nice in the color column
        john.setBackground(2, blue);
        john.setForeground(2, white);

        //do the same for the rest of the people

        TableItem jack = new TableItem(demoTable, SWT.NONE);
        String[] jackInfo = {"Jack", "64", "Red", "Pea Soup"};
        jack.setText(jackInfo);
        jack.setBackground(2, red);
        jack.setForeground(2, white);

        TableItem maria = new TableItem(demoTable, SWT.NONE);
        String[] mariaInfo = {"Maria", "22", "Pink", "Chocolate"};
        maria.setText(mariaInfo);
        maria.setBackground(2, pink);
        maria.setForeground(2, white);

        TableItem david = new TableItem(demoTable, SWT.NONE);
        String[] davidInfo = {"David", "28", "Green", "Bratwurst"};
        david.setText(davidInfo);
        david.setBackground(2, green);
        david.setForeground(2, white);

        TableItem meryl = new TableItem(demoTable, SWT.NONE);
        String[] carolInfo = {"Meryl", "53", "Blue", "Sushi"};
        meryl.setText(carolInfo);
        meryl.setBackground(2, blue);
        meryl.setForeground(2, white);

        //we have to pack each column to make it display, packing results in the
        //column calculating the required space to display all the data
        for(int i = 0; i < titles.length; i++)
        {
            demoTable.getColumn(i).pack();
        }

        demoComposite.layout();
        demoShell.open();
        demoShell.setActive();
    }

    public void widgetSelected(SelectionEvent e)
    {
        if(e.widget.equals(backCommand))
        {
            back();
        }
        else if(e.widget.equals(toggleLinesVisibleCommand))
        {
            //switch the status of the lines in the table
            linesVisible = !linesVisible;
            demoTable.setLinesVisible(linesVisible);
        }
        else if(e.widget.equals(toggleHeaderVisibleCommand))
        {
            //switch the status of the header in the table
            headerVisible = !headerVisible;
            demoTable.setHeaderVisible(headerVisible);
        }
    }

    public void widgetDefaultSelected(SelectionEvent e)
    {
    }

    private void back()
    {
        //dispose of all the colors we used
        green.dispose();
        pink.dispose();
        red.dispose();
        blue.dispose();
        white.dispose();

        demoShell.dispose();
        demoShell = null;
    }
}
